@@ -20,6 +20,7 @@ from maitai._inference import (
 from maitai._types import (
     Body,
     EvaluateCallback,
+    FallbackConfig,
     Headers,
     MaitaiChunk,
     MaitaiCompletion,
@@ -27,17 +28,22 @@ from maitai._types import (
     ToolFunction,
 )
 from maitai._utils import (
-    OpenAISchemaGenerator,
     convert_open_ai_chat_completion_chunk,
     convert_openai_chat_completion,
     get_chat_completion_params,
     required_args,
+    type_to_response_format_param,
 )
+from maitai.models.chat import (
+    AzureParams,
+    ChatCompletionParams,
+    ClientParams,
+    EvaluationContentType,
+)
+from maitai.models.config import InferenceLocations
+from maitai.models.inference import InferenceStreamResponse
+from maitai.models.metric import RequestTimingMetric
 from maitai.tools import Tools
-from maitai_gen.chat import ChatCompletionParams, ClientParams, EvaluationContentType
-from maitai_gen.config import InferenceLocations
-from maitai_gen.inference import InferenceStreamResponse
-from maitai_gen.metric import RequestTimingMetric
 
 DEFAULT_MAX_RETRIES = 2
 
@@ -52,6 +58,10 @@ class MaitaiAsync:
         groq_api_key: Optional[str] = None,
         anthropic_api_key: Optional[str] = None,
         cerebras_api_key: Optional[str] = None,
+        azure_api_key: Optional[str] = None,
+        azure_deployment: Optional[str] = None,
+        azure_api_version: Optional[str] = None,
+        azure_endpoint: Optional[str] = None,
         organization: Optional[str] = None,
         project: Optional[str] = None,
         base_url: Union[str, httpx.URL, None] = None,
@@ -65,21 +75,15 @@ class MaitaiAsync:
         if not maitai_api_key:
             maitai_api_key = os.environ.get("MAITAI_API_KEY")
         if maitai_api_key:
-            config.initialize(maitai_api_key)
+            config.initialize(maitai_api_key, use_async=True)
         else:
             raise ValueError(
                 "MAITAI_API_KEY has not been set. Either set via environment variable or pass it directly into the client."
             )
-
         if api_key:
-            if not openai_api_key:
-                openai_api_key = api_key
-            if not groq_api_key:
-                groq_api_key = api_key
-            if not anthropic_api_key:
-                anthropic_api_key = api_key
-            if not cerebras_api_key:
-                cerebras_api_key = api_key
+            raise ValueError(
+                "api_key is deprecated. Use provider specific keys instead. Supported variables: openai_api_key, groq_api_key, anthropic_api_key, cerebras_api_key, azure_api_key"
+            )
         if openai_api_key:
             config.auth_keys.openai_api_key.key_value = openai_api_key
         if groq_api_key:
@@ -88,10 +92,33 @@ class MaitaiAsync:
             config.auth_keys.anthropic_api_key.key_value = anthropic_api_key
         if cerebras_api_key:
             config.auth_keys.cerebras_api_key.key_value = cerebras_api_key
+        if azure_api_key:
+            config.auth_keys.azure_api_key.key_value = azure_api_key
         self._client = None
-        if api_key or openai_api_key or os.environ.get("OPENAI_API_KEY"):
+        if azure_api_key:
+            azure_params = AzureParams(
+                endpoint=azure_endpoint,
+                deployment=azure_deployment,
+                api_version=azure_api_version,
+            )
+            self._client = openai.AsyncAzureOpenAI(
+                azure_endpoint=azure_endpoint,
+                azure_deployment=azure_deployment,
+                api_key=azure_api_key,
+                api_version=azure_api_version,
+                organization=organization,
+                project=project,
+                base_url=base_url,
+                timeout=timeout,
+                max_retries=max_retries,
+                default_headers=default_headers,
+                default_query=default_query,
+                http_client=http_client,
+                _strict_response_validation=_strict_response_validation,
+            )
+        elif openai_api_key or os.environ.get("OPENAI_API_KEY"):
             self._client = openai.AsyncClient(
-                api_key=api_key or openai_api_key or os.environ.get("OPENAI_API_KEY"),
+                api_key=openai_api_key or os.environ.get("OPENAI_API_KEY"),
                 organization=organization,
                 project=project,
                 base_url=base_url,
@@ -109,6 +136,7 @@ class MaitaiAsync:
                 base_url=base_url or "",
                 default_headers=default_headers or {},
                 default_query=default_query or {},
+                azure_params=azure_params,
             )
         self.chat = AsyncChat(self._client, client_params)
 
@@ -156,6 +184,7 @@ class AsyncCompletions:
         assistant: bool = False,
         metadata: dict = {},
         safe_mode: bool = None,
+        fallback_config: FallbackConfig = None,
         # OpenAI Arguments
         messages: Iterable[openai_chat_types.ChatCompletionMessageParam],
         model: Union[str, openai_types.ChatModel, openai.NotGiven] = openai.NOT_GIVEN,
@@ -271,8 +300,25 @@ class AsyncCompletions:
             timeout = maitai_config.timeout
         if context_retrieval_enabled is None:
             context_retrieval_enabled = maitai_config.context_retrieval_enabled
-        if fallback_model is None and maitai_config.fallback_model:
-            fallback_model = maitai_config.fallback_model
+        if fallback_model is not None:
+            warnings.warn(
+                "fallback_model is deprecated. Use fallback_config instead.",
+                DeprecationWarning,
+            )
+        if fallback_config is None and maitai_config.fallback_config:
+            fallback_config = {
+                "model": maitai_config.fallback_config.model,
+                "strategy": maitai_config.fallback_config.strategy,
+                "timeout": maitai_config.fallback_config.timeout,
+            }
+        if fallback_config is None and fallback_model:
+            fallback_config = {
+                "model": fallback_model,
+                "strategy": "reactive",
+                "timeout": None,
+            }
+        if fallback_config and "model" in fallback_config:
+            fallback_model = fallback_config["model"]
         completion_params = get_chat_completion_params(
             messages=messages,
             model=model,
@@ -314,13 +360,13 @@ class AsyncCompletions:
                 "fallback_model": fallback_model,
                 "user_id": user_id,
                 "assistant": assistant,
-                "auth_keys": config.auth_keys.to_dict(),
+                "auth_keys": config.auth_keys.model_dump(),
                 "metadata": metadata,
                 "safe_mode": safe_mode,
             }
 
             if self._client_params is not None:
-                request_dict["client_params"] = self._client_params.to_dict()
+                request_dict["client_params"] = self._client_params.model_dump()
 
             try:
                 timing.time_pre_request = time.time()
@@ -355,9 +401,11 @@ class AsyncCompletions:
                     timing.request_id = resp.chat_completion_response.request_id
                     timing.time_request_end = time.time()
                     maitai.Inference.store_request_timing_data(timing)
-                    return MaitaiCompletion(
-                        resp.chat_completion_response, response_format=response_format
+                    completion = MaitaiCompletion.model_validate(
+                        resp.chat_completion_response.model_dump()
                     )
+                    completion.set_response_format(response_format)
+                    return completion
                 raise InferenceException("No data received")
             except MaitaiConnectionError as e:
                 if "gpt" not in model and fallback_model and "gpt" in fallback_model:
@@ -378,9 +426,7 @@ class AsyncCompletions:
             if hasattr(response_format, "model_json_schema"):
                 response_format = {
                     "type": "json_schema",
-                    "json_schema": response_format.model_json_schema(
-                        schema_generator=OpenAISchemaGenerator, mode="serialization"
-                    ),
+                    "json_schema": type_to_response_format_param(response_format),
                 }
             response = await self._client.chat.completions.create(
                 messages=messages,
@@ -460,9 +506,11 @@ class AsyncCompletions:
                         timing=timing,
                         metadata=metadata,
                     )
-                return MaitaiCompletion(
-                    maitai_completion, response_format=response_format
+                completion = MaitaiCompletion.model_validate(
+                    maitai_completion.model_dump()
                 )
+                completion.set_response_format(response_format)
+                return completion
 
 
 async def _process_async_openai_stream(
@@ -491,7 +539,7 @@ async def _process_async_openai_stream(
                 full_body += content
         if maitai_chunk.usage and last_chunk is not None and not last_chunk.usage:
             last_chunk.usage = maitai_chunk.usage
-        yield MaitaiChunk(maitai_chunk)
+        yield MaitaiChunk.model_validate(maitai_chunk.model_dump())
     timing.time_request_end = time.time()
     if last_chunk is None:
         return
@@ -545,7 +593,7 @@ async def _process_inference_stream_async(
                     timing.time_first_chunk = time.time()
                     timing.request_id = chunk.request_id
                     first = False
-                yield MaitaiChunk(chunk)
+                yield MaitaiChunk.model_validate(chunk.model_dump())
                 if chunk.choices and chunk.choices[0].finish_reason:
                     timing.time_request_end = time.time()
                     maitai.Inference.store_request_timing_data(timing)
@@ -558,8 +606,8 @@ async def _process_inference_stream_async(
                 ):
                     return
     except MaitaiConnectionError as e:
-        if "gpt" not in retry_args.get("model", "") and "gpt" in retry_args.get(
-            "fallback_model", ""
+        if "gpt" not in retry_args.get("model", "") and "gpt" in (
+            retry_args.get("fallback_model", "") or ""
         ):
             retry_args["model"] = retry_args["fallback_model"]
         if "gpt" in retry_args.get("model", ""):
