@@ -6,12 +6,12 @@ import websocket
 
 from maitai_common.processes.io_thread import IOThread
 
-logging.getLogger("websocket").setLevel(logging.WARNING)
+logging.getLogger("websocket").setLevel(logging.FATAL)
 
 
 class WebsocketListenerThread(IOThread):
-    def __init__(self, path, type, key=None):
-        super(WebsocketListenerThread, self).__init__(interval=60)
+    def __init__(self, path, type, key=None, interval=60):
+        super(WebsocketListenerThread, self).__init__(interval=interval)
         self.child_name = f"{self.__class__.__name__}"
         self.messages = []
         self.ws_url = f"{path}?type={type}"
@@ -49,7 +49,8 @@ class WebsocketListenerThread(IOThread):
         self.messages.append(message)
 
     def on_close(self, ws, _, __):
-        self.connect_to_websocket()
+        if self.run_thread and not self.closing_ws:
+            self.connect_to_websocket()
 
     def on_open(self, ws):
         self.connection_established_event.set()  # Set the event when connection is open
@@ -82,21 +83,23 @@ class WebsocketListenerThread(IOThread):
 
     def process(self):
         if self.run_thread:
-            old_ws = self.ws
-            # Temporarily override the on_close to prevent reconnection
-            old_on_close = old_ws.on_close
-            old_ws.on_close = lambda *args, **kwargs: None
+            # Check if websocket is connected and functioning
+            if not self.ws or not self.ws.sock or not self.ws.sock.connected:
+                old_ws = self.ws
+                # Temporarily override the on_close to prevent reconnection
+                old_on_close = old_ws.on_close
+                old_ws.on_close = lambda *args, **kwargs: None
 
-            try:
-                # Try to establish a new connection
-                self.connect_to_websocket()
-                # Wait for the on_open callback to set the event
-                self.connection_established_event.wait()
-                # If successful, close the old connection
-                old_ws.close()
-            except Exception as e:
-                # If new connection fails, keep the old connection
-                old_ws.on_close = old_on_close
-                self.ws = old_ws
+                try:
+                    # Try to establish a new connection
+                    self.connect_to_websocket()
+                    # Wait for the on_open callback to set the event
+                    self.connection_established_event.wait()
+                    # If successful, close the old connection
+                    old_ws.close()
+                except Exception as e:
+                    # If new connection fails, keep the old connection
+                    old_ws.on_close = old_on_close
+                    self.ws = old_ws
 
-            super(WebsocketListenerThread, self).process()
+        super(WebsocketListenerThread, self).process()
